# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.analytics.admin import gapic_version as package_version

__version__ = package_version.__version__


from google.analytics.admin_v1alpha.services.analytics_admin_service.async_client import (
    AnalyticsAdminServiceAsyncClient,
)
from google.analytics.admin_v1alpha.services.analytics_admin_service.client import (
    AnalyticsAdminServiceClient,
)
from google.analytics.admin_v1alpha.types.access_report import (
    AccessBetweenFilter,
    AccessDateRange,
    AccessDimension,
    AccessDimensionHeader,
    AccessDimensionValue,
    AccessFilter,
    AccessFilterExpression,
    AccessFilterExpressionList,
    AccessInListFilter,
    AccessMetric,
    AccessMetricHeader,
    AccessMetricValue,
    AccessNumericFilter,
    AccessOrderBy,
    AccessQuota,
    AccessQuotaStatus,
    AccessRow,
    AccessStringFilter,
    NumericValue,
)
from google.analytics.admin_v1alpha.types.analytics_admin import (
    AcknowledgeUserDataCollectionRequest,
    AcknowledgeUserDataCollectionResponse,
    ApproveDisplayVideo360AdvertiserLinkProposalRequest,
    ApproveDisplayVideo360AdvertiserLinkProposalResponse,
    ArchiveAudienceRequest,
    ArchiveCustomDimensionRequest,
    ArchiveCustomMetricRequest,
    AuditUserLinksRequest,
    AuditUserLinksResponse,
    BatchCreateUserLinksRequest,
    BatchCreateUserLinksResponse,
    BatchDeleteUserLinksRequest,
    BatchGetUserLinksRequest,
    BatchGetUserLinksResponse,
    BatchUpdateUserLinksRequest,
    BatchUpdateUserLinksResponse,
    CancelDisplayVideo360AdvertiserLinkProposalRequest,
    CreateAudienceRequest,
    CreateConversionEventRequest,
    CreateCustomDimensionRequest,
    CreateCustomMetricRequest,
    CreateDataStreamRequest,
    CreateDisplayVideo360AdvertiserLinkProposalRequest,
    CreateDisplayVideo360AdvertiserLinkRequest,
    CreateFirebaseLinkRequest,
    CreateGoogleAdsLinkRequest,
    CreateMeasurementProtocolSecretRequest,
    CreatePropertyRequest,
    CreateUserLinkRequest,
    DeleteAccountRequest,
    DeleteConversionEventRequest,
    DeleteDataStreamRequest,
    DeleteDisplayVideo360AdvertiserLinkProposalRequest,
    DeleteDisplayVideo360AdvertiserLinkRequest,
    DeleteFirebaseLinkRequest,
    DeleteGoogleAdsLinkRequest,
    DeleteMeasurementProtocolSecretRequest,
    DeletePropertyRequest,
    DeleteUserLinkRequest,
    GetAccountRequest,
    GetAttributionSettingsRequest,
    GetAudienceRequest,
    GetConversionEventRequest,
    GetCustomDimensionRequest,
    GetCustomMetricRequest,
    GetDataRetentionSettingsRequest,
    GetDataSharingSettingsRequest,
    GetDataStreamRequest,
    GetDisplayVideo360AdvertiserLinkProposalRequest,
    GetDisplayVideo360AdvertiserLinkRequest,
    GetGlobalSiteTagRequest,
    GetGoogleSignalsSettingsRequest,
    GetMeasurementProtocolSecretRequest,
    GetPropertyRequest,
    GetUserLinkRequest,
    ListAccountsRequest,
    ListAccountsResponse,
    ListAccountSummariesRequest,
    ListAccountSummariesResponse,
    ListAudiencesRequest,
    ListAudiencesResponse,
    ListConversionEventsRequest,
    ListConversionEventsResponse,
    ListCustomDimensionsRequest,
    ListCustomDimensionsResponse,
    ListCustomMetricsRequest,
    ListCustomMetricsResponse,
    ListDataStreamsRequest,
    ListDataStreamsResponse,
    ListDisplayVideo360AdvertiserLinkProposalsRequest,
    ListDisplayVideo360AdvertiserLinkProposalsResponse,
    ListDisplayVideo360AdvertiserLinksRequest,
    ListDisplayVideo360AdvertiserLinksResponse,
    ListFirebaseLinksRequest,
    ListFirebaseLinksResponse,
    ListGoogleAdsLinksRequest,
    ListGoogleAdsLinksResponse,
    ListMeasurementProtocolSecretsRequest,
    ListMeasurementProtocolSecretsResponse,
    ListPropertiesRequest,
    ListPropertiesResponse,
    ListUserLinksRequest,
    ListUserLinksResponse,
    ProvisionAccountTicketRequest,
    ProvisionAccountTicketResponse,
    RunAccessReportRequest,
    RunAccessReportResponse,
    SearchChangeHistoryEventsRequest,
    SearchChangeHistoryEventsResponse,
    UpdateAccountRequest,
    UpdateAttributionSettingsRequest,
    UpdateAudienceRequest,
    UpdateCustomDimensionRequest,
    UpdateCustomMetricRequest,
    UpdateDataRetentionSettingsRequest,
    UpdateDataStreamRequest,
    UpdateDisplayVideo360AdvertiserLinkRequest,
    UpdateGoogleAdsLinkRequest,
    UpdateGoogleSignalsSettingsRequest,
    UpdateMeasurementProtocolSecretRequest,
    UpdatePropertyRequest,
    UpdateUserLinkRequest,
)
from google.analytics.admin_v1alpha.types.audience import (
    Audience,
    AudienceDimensionOrMetricFilter,
    AudienceEventFilter,
    AudienceEventTrigger,
    AudienceFilterClause,
    AudienceFilterExpression,
    AudienceFilterExpressionList,
    AudienceFilterScope,
    AudienceSequenceFilter,
    AudienceSimpleFilter,
)
from google.analytics.admin_v1alpha.types.resources import (
    Account,
    AccountSummary,
    ActionType,
    ActorType,
    AttributionSettings,
    AuditUserLink,
    ChangeHistoryChange,
    ChangeHistoryEvent,
    ChangeHistoryResourceType,
    ConversionEvent,
    CustomDimension,
    CustomMetric,
    DataRetentionSettings,
    DataSharingSettings,
    DataStream,
    DisplayVideo360AdvertiserLink,
    DisplayVideo360AdvertiserLinkProposal,
    FirebaseLink,
    GlobalSiteTag,
    GoogleAdsLink,
    GoogleSignalsConsent,
    GoogleSignalsSettings,
    GoogleSignalsState,
    IndustryCategory,
    LinkProposalInitiatingProduct,
    LinkProposalState,
    LinkProposalStatusDetails,
    MeasurementProtocolSecret,
    Property,
    PropertySummary,
    PropertyType,
    ServiceLevel,
    UserLink,
)

__all__ = (
    "AnalyticsAdminServiceClient",
    "AnalyticsAdminServiceAsyncClient",
    "AccessBetweenFilter",
    "AccessDateRange",
    "AccessDimension",
    "AccessDimensionHeader",
    "AccessDimensionValue",
    "AccessFilter",
    "AccessFilterExpression",
    "AccessFilterExpressionList",
    "AccessInListFilter",
    "AccessMetric",
    "AccessMetricHeader",
    "AccessMetricValue",
    "AccessNumericFilter",
    "AccessOrderBy",
    "AccessQuota",
    "AccessQuotaStatus",
    "AccessRow",
    "AccessStringFilter",
    "NumericValue",
    "AcknowledgeUserDataCollectionRequest",
    "AcknowledgeUserDataCollectionResponse",
    "ApproveDisplayVideo360AdvertiserLinkProposalRequest",
    "ApproveDisplayVideo360AdvertiserLinkProposalResponse",
    "ArchiveAudienceRequest",
    "ArchiveCustomDimensionRequest",
    "ArchiveCustomMetricRequest",
    "AuditUserLinksRequest",
    "AuditUserLinksResponse",
    "BatchCreateUserLinksRequest",
    "BatchCreateUserLinksResponse",
    "BatchDeleteUserLinksRequest",
    "BatchGetUserLinksRequest",
    "BatchGetUserLinksResponse",
    "BatchUpdateUserLinksRequest",
    "BatchUpdateUserLinksResponse",
    "CancelDisplayVideo360AdvertiserLinkProposalRequest",
    "CreateAudienceRequest",
    "CreateConversionEventRequest",
    "CreateCustomDimensionRequest",
    "CreateCustomMetricRequest",
    "CreateDataStreamRequest",
    "CreateDisplayVideo360AdvertiserLinkProposalRequest",
    "CreateDisplayVideo360AdvertiserLinkRequest",
    "CreateFirebaseLinkRequest",
    "CreateGoogleAdsLinkRequest",
    "CreateMeasurementProtocolSecretRequest",
    "CreatePropertyRequest",
    "CreateUserLinkRequest",
    "DeleteAccountRequest",
    "DeleteConversionEventRequest",
    "DeleteDataStreamRequest",
    "DeleteDisplayVideo360AdvertiserLinkProposalRequest",
    "DeleteDisplayVideo360AdvertiserLinkRequest",
    "DeleteFirebaseLinkRequest",
    "DeleteGoogleAdsLinkRequest",
    "DeleteMeasurementProtocolSecretRequest",
    "DeletePropertyRequest",
    "DeleteUserLinkRequest",
    "GetAccountRequest",
    "GetAttributionSettingsRequest",
    "GetAudienceRequest",
    "GetConversionEventRequest",
    "GetCustomDimensionRequest",
    "GetCustomMetricRequest",
    "GetDataRetentionSettingsRequest",
    "GetDataSharingSettingsRequest",
    "GetDataStreamRequest",
    "GetDisplayVideo360AdvertiserLinkProposalRequest",
    "GetDisplayVideo360AdvertiserLinkRequest",
    "GetGlobalSiteTagRequest",
    "GetGoogleSignalsSettingsRequest",
    "GetMeasurementProtocolSecretRequest",
    "GetPropertyRequest",
    "GetUserLinkRequest",
    "ListAccountsRequest",
    "ListAccountsResponse",
    "ListAccountSummariesRequest",
    "ListAccountSummariesResponse",
    "ListAudiencesRequest",
    "ListAudiencesResponse",
    "ListConversionEventsRequest",
    "ListConversionEventsResponse",
    "ListCustomDimensionsRequest",
    "ListCustomDimensionsResponse",
    "ListCustomMetricsRequest",
    "ListCustomMetricsResponse",
    "ListDataStreamsRequest",
    "ListDataStreamsResponse",
    "ListDisplayVideo360AdvertiserLinkProposalsRequest",
    "ListDisplayVideo360AdvertiserLinkProposalsResponse",
    "ListDisplayVideo360AdvertiserLinksRequest",
    "ListDisplayVideo360AdvertiserLinksResponse",
    "ListFirebaseLinksRequest",
    "ListFirebaseLinksResponse",
    "ListGoogleAdsLinksRequest",
    "ListGoogleAdsLinksResponse",
    "ListMeasurementProtocolSecretsRequest",
    "ListMeasurementProtocolSecretsResponse",
    "ListPropertiesRequest",
    "ListPropertiesResponse",
    "ListUserLinksRequest",
    "ListUserLinksResponse",
    "ProvisionAccountTicketRequest",
    "ProvisionAccountTicketResponse",
    "RunAccessReportRequest",
    "RunAccessReportResponse",
    "SearchChangeHistoryEventsRequest",
    "SearchChangeHistoryEventsResponse",
    "UpdateAccountRequest",
    "UpdateAttributionSettingsRequest",
    "UpdateAudienceRequest",
    "UpdateCustomDimensionRequest",
    "UpdateCustomMetricRequest",
    "UpdateDataRetentionSettingsRequest",
    "UpdateDataStreamRequest",
    "UpdateDisplayVideo360AdvertiserLinkRequest",
    "UpdateGoogleAdsLinkRequest",
    "UpdateGoogleSignalsSettingsRequest",
    "UpdateMeasurementProtocolSecretRequest",
    "UpdatePropertyRequest",
    "UpdateUserLinkRequest",
    "Audience",
    "AudienceDimensionOrMetricFilter",
    "AudienceEventFilter",
    "AudienceEventTrigger",
    "AudienceFilterClause",
    "AudienceFilterExpression",
    "AudienceFilterExpressionList",
    "AudienceSequenceFilter",
    "AudienceSimpleFilter",
    "AudienceFilterScope",
    "Account",
    "AccountSummary",
    "AttributionSettings",
    "AuditUserLink",
    "ChangeHistoryChange",
    "ChangeHistoryEvent",
    "ConversionEvent",
    "CustomDimension",
    "CustomMetric",
    "DataRetentionSettings",
    "DataSharingSettings",
    "DataStream",
    "DisplayVideo360AdvertiserLink",
    "DisplayVideo360AdvertiserLinkProposal",
    "FirebaseLink",
    "GlobalSiteTag",
    "GoogleAdsLink",
    "GoogleSignalsSettings",
    "LinkProposalStatusDetails",
    "MeasurementProtocolSecret",
    "Property",
    "PropertySummary",
    "UserLink",
    "ActionType",
    "ActorType",
    "ChangeHistoryResourceType",
    "GoogleSignalsConsent",
    "GoogleSignalsState",
    "IndustryCategory",
    "LinkProposalInitiatingProduct",
    "LinkProposalState",
    "PropertyType",
    "ServiceLevel",
)
