# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.analytics.admin import gapic_version as package_version

__version__ = package_version.__version__


from .services.analytics_admin_service import (
    AnalyticsAdminServiceAsyncClient,
    AnalyticsAdminServiceClient,
)
from .types.analytics_admin import (
    AcknowledgeUserDataCollectionRequest,
    AcknowledgeUserDataCollectionResponse,
    ArchiveCustomDimensionRequest,
    ArchiveCustomMetricRequest,
    CreateConversionEventRequest,
    CreateCustomDimensionRequest,
    CreateCustomMetricRequest,
    CreateDataStreamRequest,
    CreateFirebaseLinkRequest,
    CreateGoogleAdsLinkRequest,
    CreateMeasurementProtocolSecretRequest,
    CreatePropertyRequest,
    DeleteAccountRequest,
    DeleteConversionEventRequest,
    DeleteDataStreamRequest,
    DeleteFirebaseLinkRequest,
    DeleteGoogleAdsLinkRequest,
    DeleteMeasurementProtocolSecretRequest,
    DeletePropertyRequest,
    GetAccountRequest,
    GetConversionEventRequest,
    GetCustomDimensionRequest,
    GetCustomMetricRequest,
    GetDataRetentionSettingsRequest,
    GetDataSharingSettingsRequest,
    GetDataStreamRequest,
    GetMeasurementProtocolSecretRequest,
    GetPropertyRequest,
    ListAccountsRequest,
    ListAccountsResponse,
    ListAccountSummariesRequest,
    ListAccountSummariesResponse,
    ListConversionEventsRequest,
    ListConversionEventsResponse,
    ListCustomDimensionsRequest,
    ListCustomDimensionsResponse,
    ListCustomMetricsRequest,
    ListCustomMetricsResponse,
    ListDataStreamsRequest,
    ListDataStreamsResponse,
    ListFirebaseLinksRequest,
    ListFirebaseLinksResponse,
    ListGoogleAdsLinksRequest,
    ListGoogleAdsLinksResponse,
    ListMeasurementProtocolSecretsRequest,
    ListMeasurementProtocolSecretsResponse,
    ListPropertiesRequest,
    ListPropertiesResponse,
    ProvisionAccountTicketRequest,
    ProvisionAccountTicketResponse,
    SearchChangeHistoryEventsRequest,
    SearchChangeHistoryEventsResponse,
    UpdateAccountRequest,
    UpdateCustomDimensionRequest,
    UpdateCustomMetricRequest,
    UpdateDataRetentionSettingsRequest,
    UpdateDataStreamRequest,
    UpdateGoogleAdsLinkRequest,
    UpdateMeasurementProtocolSecretRequest,
    UpdatePropertyRequest,
)
from .types.resources import (
    Account,
    AccountSummary,
    ActionType,
    ActorType,
    ChangeHistoryChange,
    ChangeHistoryEvent,
    ChangeHistoryResourceType,
    ConversionEvent,
    CustomDimension,
    CustomMetric,
    DataRetentionSettings,
    DataSharingSettings,
    DataStream,
    FirebaseLink,
    GoogleAdsLink,
    IndustryCategory,
    MeasurementProtocolSecret,
    Property,
    PropertySummary,
    PropertyType,
    ServiceLevel,
)

__all__ = (
    "AnalyticsAdminServiceAsyncClient",
    "Account",
    "AccountSummary",
    "AcknowledgeUserDataCollectionRequest",
    "AcknowledgeUserDataCollectionResponse",
    "ActionType",
    "ActorType",
    "AnalyticsAdminServiceClient",
    "ArchiveCustomDimensionRequest",
    "ArchiveCustomMetricRequest",
    "ChangeHistoryChange",
    "ChangeHistoryEvent",
    "ChangeHistoryResourceType",
    "ConversionEvent",
    "CreateConversionEventRequest",
    "CreateCustomDimensionRequest",
    "CreateCustomMetricRequest",
    "CreateDataStreamRequest",
    "CreateFirebaseLinkRequest",
    "CreateGoogleAdsLinkRequest",
    "CreateMeasurementProtocolSecretRequest",
    "CreatePropertyRequest",
    "CustomDimension",
    "CustomMetric",
    "DataRetentionSettings",
    "DataSharingSettings",
    "DataStream",
    "DeleteAccountRequest",
    "DeleteConversionEventRequest",
    "DeleteDataStreamRequest",
    "DeleteFirebaseLinkRequest",
    "DeleteGoogleAdsLinkRequest",
    "DeleteMeasurementProtocolSecretRequest",
    "DeletePropertyRequest",
    "FirebaseLink",
    "GetAccountRequest",
    "GetConversionEventRequest",
    "GetCustomDimensionRequest",
    "GetCustomMetricRequest",
    "GetDataRetentionSettingsRequest",
    "GetDataSharingSettingsRequest",
    "GetDataStreamRequest",
    "GetMeasurementProtocolSecretRequest",
    "GetPropertyRequest",
    "GoogleAdsLink",
    "IndustryCategory",
    "ListAccountSummariesRequest",
    "ListAccountSummariesResponse",
    "ListAccountsRequest",
    "ListAccountsResponse",
    "ListConversionEventsRequest",
    "ListConversionEventsResponse",
    "ListCustomDimensionsRequest",
    "ListCustomDimensionsResponse",
    "ListCustomMetricsRequest",
    "ListCustomMetricsResponse",
    "ListDataStreamsRequest",
    "ListDataStreamsResponse",
    "ListFirebaseLinksRequest",
    "ListFirebaseLinksResponse",
    "ListGoogleAdsLinksRequest",
    "ListGoogleAdsLinksResponse",
    "ListMeasurementProtocolSecretsRequest",
    "ListMeasurementProtocolSecretsResponse",
    "ListPropertiesRequest",
    "ListPropertiesResponse",
    "MeasurementProtocolSecret",
    "Property",
    "PropertySummary",
    "PropertyType",
    "ProvisionAccountTicketRequest",
    "ProvisionAccountTicketResponse",
    "SearchChangeHistoryEventsRequest",
    "SearchChangeHistoryEventsResponse",
    "ServiceLevel",
    "UpdateAccountRequest",
    "UpdateCustomDimensionRequest",
    "UpdateCustomMetricRequest",
    "UpdateDataRetentionSettingsRequest",
    "UpdateDataStreamRequest",
    "UpdateGoogleAdsLinkRequest",
    "UpdateMeasurementProtocolSecretRequest",
    "UpdatePropertyRequest",
)
